@@ -13,9 +13,10 @@ from pex.compatibility import indent
 from pex.dist_metadata import Requirement, RequirementParseError
 from pex.enum import Enum
 from pex.interpreter import PythonInterpreter
+from pex.interpreter_implementation import InterpreterImplementation
 from pex.orderedset import OrderedSet
 from pex.specifier_sets import UnsatisfiableSpecifierSet, as_range
-from pex.third_party.packaging.specifiers import SpecifierSet
+from pex.third_party.packaging.specifiers import InvalidSpecifier, SpecifierSet
 from pex.typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
@@ -38,23 +39,19 @@ class InterpreterConstraint(object):
     def parse(
         cls,
         constraint,  # type: str
-        default_interpreter=None,  # type: Optional[str]
+        default_interpreter=None,  # type: Optional[InterpreterImplementation.Value]
     ):
         # type: (...) -> InterpreterConstraint
         try:
             requirement = Requirement.parse(constraint)
-            return cls(specifier=requirement.specifier, name=requirement.name)
+            return cls(
+                specifier=requirement.specifier,
+                implementation=InterpreterImplementation.for_value(requirement.name),
+            )
         except RequirementParseError:
-            if default_interpreter is not None:
-                return cls.parse(
-                    constraint="{interpreter}{specifier}".format(
-                        interpreter=default_interpreter, specifier=constraint
-                    ),
-                    default_interpreter=None,
-                )
             try:
-                return cls(specifier=SpecifierSet(constraint))
-            except RequirementParseError as e:
+                return cls(specifier=SpecifierSet(constraint), implementation=default_interpreter)
+            except InvalidSpecifier as e:
                 raise ValueError(
                     "Unparseable interpreter constraint {constraint}: {err}".format(
                         constraint=constraint, err=e
@@ -74,13 +71,13 @@ class InterpreterConstraint(object):
         # type: (Optional[PythonInterpreter]) -> InterpreterConstraint
         python_identity = (interpreter or PythonInterpreter.get()).identity
         return cls.parse(
-            "{interpreter}=={version}".format(
-                interpreter=python_identity.interpreter, version=python_identity.version_str
+            "{implementation}=={version}".format(
+                implementation=python_identity.implementation, version=python_identity.version_str
             )
         )
 
     specifier = attr.ib()  # type: SpecifierSet
-    name = attr.ib(default=None)  # type: Optional[str]
+    implementation = attr.ib(default=None)  # type: Optional[InterpreterImplementation.Value]
 
     @specifier.validator
     def _validate_specifier(
@@ -107,12 +104,14 @@ class InterpreterConstraint(object):
 
     def __str__(self):
         # type: () -> str
-        return "{name}{specifier}".format(name=self.name or "", specifier=self.specifier)
+        return "{implementation}{specifier}".format(
+            implementation=self.implementation or "", specifier=self.specifier
+        )
 
     def __contains__(self, interpreter):
         # type: (PythonInterpreter) -> bool
         python_identity = interpreter.identity
-        if self.name and self.name != python_identity.interpreter:
+        if self.implementation and self.implementation is not python_identity.implementation:
             return False
         return python_identity.version_str in self.specifier
 
@@ -380,18 +379,18 @@ COMPATIBLE_PYTHON_VERSIONS = (
     PythonVersion(Lifecycle.STABLE, 3, 10, 18),
     PythonVersion(Lifecycle.STABLE, 3, 11, 13),
     PythonVersion(Lifecycle.STABLE, 3, 12, 11),
-    PythonVersion(Lifecycle.STABLE, 3, 13, 5),
+    PythonVersion(Lifecycle.STABLE, 3, 13, 7),
     PythonVersion(Lifecycle.DEV, 3, 14, 0),
 )
 
 
 def iter_compatible_versions(
-    requires_python,  # type: Iterable[str]
+    requires_python,  # type: Iterable[SpecifierSet]
     max_patch=DEFAULT_MAX_PATCH,  # type: int
 ):
     # type: (...) -> Iterator[Tuple[int, int, int]]
 
-    specifier_sets = OrderedSet(SpecifierSet(req) for req in requires_python)
+    specifier_sets = OrderedSet(requires_python)
     return itertools.chain.from_iterable(
         python_version.iter_compatible_versions(specifier_sets, max_patch=max_patch)
         for python_version in COMPATIBLE_PYTHON_VERSIONS
