@@ -15,15 +15,20 @@ from pex.dependency_configuration import DependencyConfiguration
 from pex.dist_metadata import DistMetadata, Requirement, is_sdist, is_wheel
 from pex.enum import Enum
 from pex.exceptions import production_assert
+from pex.interpreter_constraints import InterpreterConstraint
+from pex.interpreter_implementation import InterpreterImplementation
 from pex.orderedset import OrderedSet
 from pex.pep_425 import CompatibilityTags, TagRank
+from pex.pep_440 import Version
 from pex.pep_503 import ProjectName
 from pex.rank import Rank
 from pex.resolve.resolved_requirement import PartialArtifact, Pin, ResolvedRequirement
 from pex.resolve.resolver_configuration import BuildConfiguration
+from pex.resolve.target_system import TargetSystem, UniversalTarget
 from pex.result import Error
 from pex.sorted_tuple import SortedTuple
 from pex.targets import Target
+from pex.third_party.packaging.markers import Marker
 from pex.tracer import TRACER
 from pex.typing import TYPE_CHECKING, Generic
 
@@ -67,33 +72,71 @@ class LockStyle(Enum["LockStyle.Value"]):
 LockStyle.seal()
 
 
-class TargetSystem(Enum["TargetSystem.Value"]):
-    class Value(Enum.Value):
-        pass
-
-    LINUX = Value("linux")
-    MAC = Value("mac")
-    WINDOWS = Value("windows")
-
-
-TargetSystem.seal()
-
-
 @attr.s(frozen=True)
 class LockConfiguration(object):
+    @classmethod
+    def universal(
+        cls,
+        interpreter_constraints=(),  # type: Iterable[InterpreterConstraint]
+        systems=(),  # type: Iterable[TargetSystem.Value]
+        elide_unused_requires_dist=False,  # type: bool
+    ):
+        # type: (...) -> LockConfiguration
+
+        implementations = defaultdict(
+            set
+        )  # type: DefaultDict[Optional[InterpreterImplementation.Value], Set[SpecifierSet]]
+        for interpreter_constraint in interpreter_constraints:
+            implementations[interpreter_constraint.implementation].add(
+                interpreter_constraint.requires_python
+            )
+        if len(implementations) > 1 and frozenset(implementations) != frozenset(
+            InterpreterImplementation.values()
+        ):
+            raise ValueError(
+                "The interpreter constraints for a universal resolve cannot have mixed "
+                "implementations unless they are all explicit and span the full set of Pex "
+                "supported implementations: {implementations}.\n"
+                "Given: {constraints}".format(
+                    implementations=" and ".join(map(str, InterpreterImplementation.values())),
+                    constraints=" or ".join(map(str, interpreter_constraints)),
+                )
+            )
+        elif len(set(map(frozenset, implementations.values()))) > 1:
+            raise ValueError(
+                "The interpreter constraints for a universal resolve cannot have mixed "
+                "implementations with differing specifiers.\n"
+                "If you feel you need this support, please comment here: "
+                "https://github.com/pex-tool/pex/issues/2897#issuecomment-3256619591\n"
+                "Given: {constraints}".format(
+                    constraints=" or ".join(map(str, interpreter_constraints))
+                )
+            )
+
+        return cls(
+            style=LockStyle.UNIVERSAL,
+            universal_target=UniversalTarget(
+                implementation=next(iter(implementations)) if len(implementations) == 1 else None,
+                requires_python=tuple(
+                    interpreter_constraint.requires_python
+                    for interpreter_constraint in interpreter_constraints
+                ),
+                systems=tuple(systems),
+            ),
+            elide_unused_requires_dist=elide_unused_requires_dist,
+        )
+
     style = attr.ib()  # type: LockStyle.Value
-    requires_python = attr.ib(default=())  # type: Tuple[str, ...]
-    target_systems = attr.ib(default=())  # type: Tuple[TargetSystem.Value, ...]
+    universal_target = attr.ib(default=None)  # type: Optional[UniversalTarget]
     elide_unused_requires_dist = attr.ib(default=False)  # type: bool
 
-    @requires_python.validator
-    @target_systems.validator
+    @universal_target.validator
     def _validate_only_set_for_universal(
         self,
         attribute,  # type: Any
         value,  # type: Any
     ):
-        if value and self.style != LockStyle.UNIVERSAL:
+        if value and self.style is not LockStyle.UNIVERSAL:
             raise ValueError(
                 "The {field_name} field should only be set for {universal} style locks; "
                 "this lock is {style} style and given {field_name} value of {value}".format(
@@ -238,7 +281,11 @@ class VCSArtifact(Artifact):
     def split_requested_revision(artifact_url):
         # type: (ArtifactURL) -> Tuple[ArtifactURL, Optional[str]]
 
-        vcs_url, _, requested_revision = artifact_url.normalized_url.partition("@")
+        vcs_url = artifact_url.normalized_url
+        if "@" in artifact_url.path:
+            vcs_url, _, requested_revision = vcs_url.rpartition("@")
+        else:
+            requested_revision = None
         return ArtifactURL.parse(vcs_url), requested_revision or None
 
     @classmethod
@@ -541,7 +588,7 @@ class DownloadableArtifact(object):
         attr.ib()
     )  # type: Union[FileArtifact, LocalProjectArtifact, UnFingerprintedLocalProjectArtifact, UnFingerprintedVCSArtifact, VCSArtifact]
     satisfied_direct_requirements = attr.ib(default=SortedTuple())  # type: SortedTuple[Requirement]
-    version = attr.ib(default=None)
+    version = attr.ib(default=None)  # type: Optional[Version]
 
     def specifier(self):
         # type: () -> str
@@ -616,6 +663,7 @@ class LockedResolve(object):
         dist_metadatas,  # type: Iterable[DistMetadata]
         fingerprinter,  # type: Fingerprinter
         platform_tag=None,  # type: Optional[tags.Tag]
+        marker=None,  # type: Optional[Marker]
     ):
         # type: (...) -> LockedResolve
 
@@ -685,15 +733,24 @@ class LockedResolve(object):
                     ),
                 )
             )
-        return cls(locked_requirements=SortedTuple(locked_requirements), platform_tag=platform_tag)
+        return cls(
+            locked_requirements=SortedTuple(locked_requirements),
+            platform_tag=platform_tag,
+            marker=marker,
+        )
 
     locked_requirements = attr.ib()  # type: SortedTuple[LockedRequirement]
     platform_tag = attr.ib(order=str, default=None)  # type: Optional[tags.Tag]
+    marker = attr.ib(order=str, default=None)  # type: Optional[Marker]
 
     @property
     def target_platform(self):
         # type: () -> str
-        return str(self.platform_tag) if self.platform_tag else "universal"
+        if self.platform_tag:
+            return str(self.platform_tag)
+        if self.marker:
+            return "universal {marker}".format(marker=self.marker)
+        return "universal"
 
     def create_resolved_artifacts(
         self,
