@@ -3,17 +3,16 @@
 
 from __future__ import absolute_import
 
-import itertools
-
 from pex import pex_warnings
-from pex.auth import PasswordEntry
 from pex.enum import Enum
 from pex.jobs import DEFAULT_MAX_JOBS
 from pex.network_configuration import NetworkConfiguration
 from pex.pep_440 import Version
 from pex.pep_503 import ProjectName
 from pex.pip.version import PipVersion, PipVersionValue
+from pex.resolve.package_repository import ReposConfiguration
 from pex.typing import TYPE_CHECKING
+from pex.venv.virtualenv import Virtualenv
 
 if TYPE_CHECKING:
     from typing import Callable, FrozenSet, Iterable, Optional, Tuple, Union
@@ -26,9 +25,6 @@ else:
     from pex.third_party import attr
 
 
-PYPI = "https://pypi.org/simple"
-
-
 class ResolverVersion(Enum["ResolverVersion.Value"]):
     class Value(Enum.Value):
         pass
@@ -60,32 +56,6 @@ class ResolverVersion(Enum["ResolverVersion.Value"]):
 ResolverVersion.seal()
 
 
-@attr.s(frozen=True)
-class ReposConfiguration(object):
-    @classmethod
-    def create(
-        cls,
-        indexes=(),  # type: Iterable[str]
-        find_links=(),  # type: Iterable[str]
-    ):
-        # type: (...) -> ReposConfiguration
-        password_entries = []
-        for url in itertools.chain(indexes, find_links):
-            password_entry = PasswordEntry.maybe_extract_from_url(url)
-            if password_entry:
-                password_entries.append(password_entry)
-
-        return cls(
-            indexes=tuple(indexes),
-            find_links=tuple(find_links),
-            password_entries=tuple(password_entries),
-        )
-
-    indexes = attr.ib(default=(PYPI,))  # type: Tuple[str, ...]
-    find_links = attr.ib(default=())  # type: Tuple[str, ...]
-    password_entries = attr.ib(default=())  # type: Tuple[PasswordEntry, ...]
-
-
 @attr.s(frozen=True)
 class BuildConfiguration(object):
     class Error(ValueError):
@@ -277,3 +247,19 @@ class PreResolvedConfiguration(object):
     def network_configuration(self):
         # type: () -> NetworkConfiguration
         return self.pip_configuration.network_configuration
+
+
+@attr.s(frozen=True)
+class VenvRepositoryConfiguration(object):
+    venv = attr.ib()  # type: Virtualenv
+    pip_configuration = attr.ib()  # type: PipConfiguration
+
+    @property
+    def repos_configuration(self):
+        # type: () -> ReposConfiguration
+        return self.pip_configuration.repos_configuration
+
+    @property
+    def network_configuration(self):
+        # type: () -> NetworkConfiguration
+        return self.pip_configuration.network_configuration
