@@ -66,7 +66,7 @@ class Manifest(object):
 
 
 SCIENCE_RELEASES_URL = "https://github.com/a-scie/lift/releases"
-MIN_SCIENCE_VERSION = Version("0.12.8")
+MIN_SCIENCE_VERSION = Version("0.13.0")
 SCIENCE_REQUIREMENT = SpecifierSet("~={min_version}".format(min_version=MIN_SCIENCE_VERSION))
 
 
@@ -80,8 +80,8 @@ def _science_binary_url(suffix=""):
     )
 
 
-PTEX_VERSION = "1.6.1"
-SCIE_JUMP_VERSION = "1.7.0"
+PTEX_VERSION = "1.7.0"
+SCIE_JUMP_VERSION = "1.8.0"
 
 
 class Filenames(Enum["Filenames.Value"]):
@@ -275,9 +275,9 @@ def create_manifests(
 
         if configuration.options.base:
             lift["base"] = configuration.options.base
-        elif pex_info.pex_root:
+        elif pex_info.pex_root_set:
             lift["base"] = CacheDir.SCIES.path(
-                "base", os=interpreter.platform.os, pex_root=pex_info.pex_root
+                "base", os=interpreter.platform.os, pex_root=pex_info.raw_pex_root
             )
 
         manifest_path = os.path.join(
