@@ -1,4 +1,4 @@
-# Copyright 2015 Pants project contributors (see CONTRIBUTORS.md).
+# Copyright 2015 Pex project contributors.
 # Licensed under the Apache License, Version 2.0 (see LICENSE).
 
 # Due to the PEX_ properties, disable checkstyle.
@@ -14,19 +14,24 @@ import sys
 from contextlib import contextmanager
 from textwrap import dedent
 
-from pex import pex_warnings
-from pex.common import can_write_dir, die, safe_mkdtemp
+from pex import pex_root, pex_warnings
+from pex.common import die
 from pex.inherit_path import InheritPath
+from pex.orderedset import OrderedSet
 from pex.typing import TYPE_CHECKING, Generic, overload
 from pex.venv.bin_path import BinPath
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, Dict, Iterator, Optional, Tuple, Type, TypeVar, Union
+    from typing import Any, Callable, Dict, Iterator, Mapping, Optional, Tuple, Type, TypeVar, Union
 
     _O = TypeVar("_O")
     _P = TypeVar("_P")
 
-    # N.B.: This is an expensive import and we only need it for type checking.
+    # N.B.: This import is circular, and we import lazily below as a result, but we also need the
+    # import eagerly for type checking.
+    from pex.cache.dirs import UnzipDir, VenvDir  # noqa
+
+    # N.B.: This is an expensive import, and we only need it for type checking.
     from pex.interpreter import PythonInterpreter
 
 
@@ -48,11 +53,11 @@ class DefaultedProperty(Generic["_O", "_P"]):
     def __init__(
         self,
         func,  # type: Callable[[_O], _P]
-        default,  # type: _P
+        default,  # type: Union[_P, Callable[[], _P]]
     ):
         # type: (...) -> None
         self._func = func  # type: Callable[[_O], _P]
-        self._default = default  # type: _P
+        self._default = default  # type: Union[_P, Callable[[], _P]]
         self._validator = None  # type: Optional[Callable[[_O, _P], _P]]
 
     @overload
@@ -84,7 +89,9 @@ class DefaultedProperty(Generic["_O", "_P"]):
         try:
             return self._validate(instance, self._func(instance))
         except NoValueError:
-            return self._validate(instance, self._default)
+            return self._validate(
+                instance, self._default() if callable(self._default) else self._default
+            )
 
     def strip_default(self, instance):
         # type: (_O) -> Optional[_P]
@@ -139,7 +146,8 @@ class DefaultedProperty(Generic["_O", "_P"]):
 
 
 def defaulted_property(
-    default,  # type: _P
+    default,  # type: Union[_P, Callable[[], _P]]
+    _type_hint=None,  # type: Optional[Type[_P]]  # N.B.: Can be used to help MyPy along.
 ):
     # type: (...) -> Callable[[Callable[[_O], _P]], DefaultedProperty[_O, _P]]
     """Creates a `@property` with a `default` value.
@@ -155,6 +163,17 @@ def defaulted_property(
     return wrapped
 
 
+def _default_pex_root():
+    # type: () -> str
+
+    # N.B.: We need lazy import gymnastics here since cache uses appdirs which is pex.third_party
+    # and the pex.third_party mechanism uses ENV.PEX_VERBOSE indirectly via TRACER to log certain
+    # third party import lifecycle events.
+    from pex.cache import root as cache_root
+
+    return cache_root.path(expand_user=True)
+
+
 class Variables(object):
     """Environment variables supported by the PEX runtime."""
 
@@ -188,8 +207,8 @@ class Variables(object):
         """
         ret_vars = {}  # type: Dict[str, str]
         rc_locations = [
-            "/etc/pexrc",
-            "~/.pexrc",
+            os.path.join(os.sep, "etc", "pexrc"),
+            os.path.join("~", ".pexrc"),
             os.path.join(os.path.dirname(sys.argv[0]), ".pexrc"),
         ]
         if rc:
@@ -209,15 +228,40 @@ class Variables(object):
         if len(list(filter(None, kv))) == 2:
             return kv
 
+    @staticmethod
+    def _maybe_get_bool_var(
+        name,  # type: str
+        env,  # type: Mapping[str, Optional[str]]
+    ):
+        # type: (...) -> Optional[bool]
+        value = env.get(name, None)
+        if value is None:
+            return None
+        if value.lower() in ("0", "false"):
+            return False
+        if value.lower() in ("1", "true"):
+            return True
+        raise ValueError(
+            "Invalid bool value for {name}, must be 0/1/false/true, got {value!r}".format(
+                name=name, value=value
+            )
+        )
+
     def __init__(self, environ=None, rc=None):
-        # type: (Optional[Dict[str, str]], Optional[str]) -> None
-        self._environ = (
-            environ.copy() if environ is not None else os.environ.copy()
-        )  # type: Dict[str, str]
-        if not self.PEX_IGNORE_RCFILES:
-            rc_values = self.from_rc(rc).copy()
-            rc_values.update(self._environ)
-            self._environ = rc_values
+        # type: (Optional[Mapping[str, str]], Optional[str]) -> None
+        env = environ if environ is not None else os.environ
+        if self._maybe_get_bool_var("PEX_DISABLE_VARIABLES", env) is True:
+            self._environ = {
+                key: value
+                for key, value in env.items()
+                if key == "PEX_DISABLE_VARIABLES" or not key.startswith("PEX_")
+            }
+        else:
+            self._environ = dict(env)
+            if not self.PEX_IGNORE_RCFILES:
+                rc_values = self.from_rc(rc).copy()
+                rc_values.update(self._environ)
+                self._environ = rc_values
 
         if "PEX_ALWAYS_CACHE" in self._environ:
             pex_warnings.warn(
@@ -235,6 +279,12 @@ class Variables(object):
                 "The `PEX_UNZIP` env var is deprecated. This env var is no longer read since "
                 "unzipping PEX zip files before execution is now the default."
             )
+        if "PEX_TEARDOWN_VERBOSE" in self._environ:
+            pex_warnings.warn(
+                "The `PEX_TEARDOWN_VERBOSE` env var is deprecated. This env var is no longer read "
+                "since PEX teardown has been removed in favor of the natural teardown environment "
+                "provided by the Python runtime."
+            )
 
     def copy(self):
         # type: () -> Dict[str, str]
@@ -253,14 +303,7 @@ class Variables(object):
 
     def _maybe_get_bool(self, variable):
         # type: (str) -> Optional[bool]
-        value = self._maybe_get_string(variable)
-        if value is None:
-            return None
-        if value.lower() in ("0", "false"):
-            return False
-        if value.lower() in ("1", "true"):
-            return True
-        die("Invalid value for %s, must be 0/1/false/true, got %r" % (variable, value))
+        return self._maybe_get_bool_var(variable, self._environ)
 
     def _get_bool(self, variable):
         # type: (str) -> bool
@@ -294,10 +337,27 @@ class Variables(object):
                 % (variable, self._environ[variable])
             )
 
+    def _maybe_get_path_tuple(
+        self,
+        variable,  # type: str
+        empty_string_is_cwd=True,  # type: bool
+    ):
+        # type: (...) -> Optional[Tuple[str, ...]]
+        value = self._maybe_get_string(variable)
+        if value is None:
+            return None
+        if not value and not empty_string_is_cwd:
+            return None
+        return tuple(
+            OrderedSet(os.path.normpath(os.path.expanduser(p)) for p in value.split(os.pathsep))
+        )
+
     def strip(self):
         # type: () -> Variables
         stripped_environ = {
-            k: v for k, v in self.copy().items() if not k.startswith(("PEX_", "__PEX_"))
+            k: v
+            for k, v in self.copy().items()
+            if k.startswith("__PEX_BUILD_") or not k.startswith(("PEX_", "__PEX_"))
         }
         return Variables(environ=stripped_environ)
 
@@ -310,16 +370,37 @@ class Variables(object):
         present. The rest will be added to the environment or else updated if already present in
         the environment.
         """
+        disable_env = self._maybe_get_bool_var("PEX_DISABLE_VARIABLES", kw)
+
         old_environ = self._environ
         self._environ = self._environ.copy()
+        if disable_env:
+            for k in list(self._environ):
+                if k != "PEX_DISABLE_VARIABLES" and k.startswith("PEX_"):
+                    self._environ.pop(k)
         for k, v in kw.items():
             if v is None:
                 self._environ.pop(k, None)
+            elif disable_env and k != "PEX_DISABLE_VARIABLES" and k.startswith("PEX_"):
+                self._environ.pop(k, None)
             else:
                 self._environ[k] = v
+
         yield self._environ
         self._environ = old_environ
 
+    @defaulted_property(default=False)
+    def PEX_DISABLE_VARIABLES(self):
+        # type: () -> bool
+        """Boolean.
+
+        Disable reading of all PEX_* variables (except this one) from all sources. Both PEX_*
+        environment variables and PEX_* variables sources from pexrc files will be ignored.
+
+        This can be used to lock down the function of a PEX.
+        """
+        return self._get_bool("PEX_DISABLE_VARIABLES")
+
     @property
     def PEX(self):
         # type: () -> Optional[str]
@@ -459,6 +540,30 @@ class Variables(object):
         """
         return self._get_bool("PEX_INTERPRETER")
 
+    @defaulted_property(default=False)
+    def PEX_INTERPRETER_HISTORY(self):
+        # type: () -> bool
+        """Boolean.
+
+        IF PEX_INTERPRETER is true, use a command history file for REPL user convenience.
+        The location of the history file is determined by PEX_INTERPRETER_HISTORY_FILE.
+
+        Default: false.
+        """
+        return self._get_bool("PEX_INTERPRETER_HISTORY")
+
+    @defaulted_property(default=os.path.join("~", ".python_history"))
+    def PEX_INTERPRETER_HISTORY_FILE(self):
+        # type: () -> str
+        """File.
+
+        IF PEX_INTERPRETER_HISTORY is true, use this history file.
+        The default is the standard Python interpreter history location.
+
+        Default: ~/.python_history.
+        """
+        return self._get_string("PEX_INTERPRETER_HISTORY_FILE")
+
     @property
     def PEX_MODULE(self):
         # type: () -> Optional[str]
@@ -513,76 +618,67 @@ class Variables(object):
 
     @property
     def PEX_PYTHON_PATH(self):
-        # type: () -> Optional[str]
+        # type: () -> Optional[Tuple[str, ...]]
         """String.
 
-        A colon-separated string containing paths of blessed Python interpreters
-        for overriding the Python interpreter used to invoke this PEX. Can be absolute paths to
-        interpreters or standard $PATH style directory entries that are searched for child files that
-        are python binaries.
+        A ':' or ';' separated string containing paths of blessed Python interpreters for
+        overriding the Python interpreter used to invoke this PEX. Can be absolute paths to
+        interpreters or standard $PATH style directory entries that are searched for child files
+        that are python binaries.
 
-        Ex: "/path/to/python27:/path/to/python36-distribution/bin"
+        For example, on a Unix system: "/path/to/python27:/path/to/python36-distribution/bin"
         """
-        return self._maybe_get_string("PEX_PYTHON_PATH")
+        return self._maybe_get_path_tuple("PEX_PYTHON_PATH")
 
     @property
     def PEX_EXTRA_SYS_PATH(self):
-        # type: () -> Optional[str]
+        # type: () -> Tuple[str, ...]
         """String.
 
-        A colon-separated string containing paths to add to the runtime sys.path.
+        A ':' or ';' separated string containing paths to add to the runtime sys.path.
 
         Should be used sparingly, e.g., if you know that code inside this PEX needs to
         interact with code outside it.
 
-        Ex: "/path/to/lib1:/path/to/lib2"
+        For example, on a Unix system: "/path/to/lib1:/path/to/lib2"
 
         This is distinct from PEX_INHERIT_PATH, which controls how the interpreter's
         existing sys.path (which you may not have control over) is scrubbed.
 
         See also PEX_PATH for how to merge packages from other pexes into the current environment.
         """
-        return self._maybe_get_string("PEX_EXTRA_SYS_PATH")
+        return self._maybe_get_path_tuple("PEX_EXTRA_SYS_PATH") or ()
 
-    @defaulted_property(default="~/.pex")
+    @defaulted_property(default=_default_pex_root, _type_hint=str)
     def PEX_ROOT(self):
         # type: () -> str
         """Directory.
 
-        The directory location for PEX to cache any dependencies and code.  PEX must write not-zip-
-        safe eggs and all wheels to disk in order to activate them.
-
-        Default: ~/.pex
+        The directory location for PEX to cache any dependencies and code.
         """
         return self._get_path("PEX_ROOT")
 
     @PEX_ROOT.validator
     def _ensure_writeable_pex_root(self, raw_pex_root):
-        pex_root = os.path.expanduser(raw_pex_root)
-        if not can_write_dir(pex_root):
-            tmp_root = os.path.realpath(safe_mkdtemp())
-            pex_warnings.warn(
-                "PEX_ROOT is configured as {pex_root} but that path is un-writeable, "
-                "falling back to a temporary PEX_ROOT of {tmp_root} which will hurt "
-                "performance.".format(pex_root=pex_root, tmp_root=tmp_root)
-            )
-            pex_root = self._environ["PEX_ROOT"] = tmp_root
-        return pex_root
+        writeable_pex_root, is_fallback = pex_root.ensure_writeable(raw_pex_root)
+        if is_fallback:
+            self._environ["PEX_ROOT"] = writeable_pex_root
+        return writeable_pex_root
 
-    @defaulted_property(default="")
+    @property
     def PEX_PATH(self):
-        # type: () -> str
+        # type: () -> Tuple[str, ...]
         """A set of one or more PEX files.
 
         Merge the packages from other PEX files into the current environment.  This allows you to
         do things such as create a PEX file containing the "coverage" module or create PEX files
         containing plugin entry points to be consumed by a main application.  Paths should be
-        specified in the same manner as $PATH, e.g. PEX_PATH=/path/to/pex1.pex:/path/to/pex2.pex
-        and so forth.
+        specified in the same manner as $PATH. For example, on a Unix system
+        PEX_PATH=/path/to/pex1.pex:/path/to/pex2.pex and so forth.
 
         See also PEX_EXTRA_SYS_PATH for how to add arbitrary entries to the sys.path.
         """
-        return self._get_string("PEX_PATH")
+        return self._maybe_get_path_tuple("PEX_PATH", empty_string_is_cwd=False) or ()
 
     @property
     def PEX_SCRIPT(self):
@@ -655,6 +751,37 @@ class Variables(object):
         """
         return self._get_bool("PEX_TOOLS")
 
+    @defaulted_property(default=1)
+    def PEX_MAX_INSTALL_JOBS(self):
+        # type: () -> int
+        """Integer.
+
+        The maximum number of parallel jobs to use when installing third party dependencies
+        contained in a PEX during its first boot. Values are interpreted as follows:
+
+        * ``>=2`` Dependencies should be installed in parallel using exactly this maximum number of
+          jobs.
+        * ``1`` Dependencies should be installed in serial.
+        * ``0`` The maximum number of parallel jobs should be auto-selected taking the number of
+          cores into account.
+        * ``-1`` The maximum number of parallel jobs should be auto-selected taking both the
+          characteristics of the third party dependencies contained in the PEX and the number of
+          cores into account. The third party dependency heuristics are intended to yield good
+          install performance, but are opaque and may change across PEX releases if better
+          heuristics are discovered.
+        * ``<=-2`` These are illegal values; an error is raised.
+
+        Default: 1
+        """
+        install_jobs = self._get_int("PEX_MAX_INSTALL_JOBS")
+        if install_jobs < -1:
+            raise ValueError(
+                "PEX_MAX_INSTALL_JOBS must be -1 or greater; given: {jobs}".format(
+                    jobs=install_jobs
+                )
+            )
+        return install_jobs
+
     def __repr__(self):
         return "{}({!r})".format(type(self).__name__, self._environ)
 
@@ -664,30 +791,40 @@ ENV = Variables()
 
 
 # TODO(John Sirois): Extract a runtime.modes package to hold code dealing with runtime mode
-#  calculations: https://github.com/pantsbuild/pex/issues/1154
+#  calculations: https://github.com/pex-tool/pex/issues/1154
 def _expand_pex_root(pex_root):
     # type: (str) -> str
-    fallback = os.path.expanduser(pex_root)
+    fallback = os.path.realpath(os.path.expanduser(pex_root))
     return os.path.expanduser(Variables.PEX_ROOT.value_or(ENV, fallback=fallback))
 
 
 def unzip_dir(
     pex_root,  # type: str
     pex_hash,  # type: str
+    expand_pex_root=True,  # type: bool
 ):
-    # type: (...) -> str
-    return os.path.join(_expand_pex_root(pex_root), "unzipped_pexes", pex_hash)
+    # type: (...) -> UnzipDir
+
+    # N.B.: We need lazy import gymnastics here since CacheType uses Variables for PEX_ROOT.
+    from pex.cache.dirs import UnzipDir
+
+    pex_root = _expand_pex_root(pex_root) if expand_pex_root else pex_root
+    return UnzipDir.create(pex_hash=pex_hash, pex_root=pex_root)
 
 
 def venv_dir(
-    pex_file,  # type: str
     pex_root,  # type: str
     pex_hash,  # type: str
     has_interpreter_constraints,  # type: bool
+    pex_file=None,  # type: Optional[str]
     interpreter=None,  # type: Optional[PythonInterpreter]
-    pex_path=None,  # type: Optional[str]
+    pex_path=(),  # type: Tuple[str, ...]
+    expand_pex_root=True,  # type: bool
 ):
-    # type: (...) -> str
+    # type: (...) -> VenvDir
+
+    # N.B.: We need lazy import gymnastics here since CacheType uses Variables for PEX_ROOT.
+    from pex.cache.dirs import VenvDir
 
     # The venv contents are affected by which PEX files are in play as well as which interpreter
     # is selected. The former is influenced via PEX_PATH and the latter is influenced by interpreter
@@ -699,13 +836,13 @@ def venv_dir(
     # PexInfo.pex_path and PEX_PATH are merged by PEX at runtime, so we include both and hash just
     # the distributions since those are the only items used from PEX_PATH adjoined PEX files; i.e.:
     # neither the entry_point nor any other PEX file data or metadata is used.
-    def add_pex_path_items(path):
-        # type: (Optional[str]) -> None
-        if not path:
+    def add_pex_path_items(pexes):
+        # type: (Tuple[str, ...]) -> None
+        if not pexes:
             return
         from pex.pex_info import PexInfo
 
-        for pex in path.split(":"):
+        for pex in pexes:
             pex_path_contents[pex] = PexInfo.from_pex(pex).distributions
 
     add_pex_path_items(pex_path)
@@ -735,11 +872,12 @@ def venv_dir(
         # Otherwise we can calculate the exact interpreter the PEX runtime will use. This ensures
         # ~unconstrained PEXes get a venv per interpreter used to invoke them with (user chooses).
         interpreter_binary = interpreter.binary if interpreter else sys.executable
-        pex_python_path = tuple(ENV.PEX_PYTHON_PATH.split(":")) if ENV.PEX_PYTHON_PATH else None
         if (
-            not pex_python_path
-            or interpreter_binary.startswith(pex_python_path)
-            or os.path.realpath(interpreter_binary).startswith(pex_python_path)
+            not ENV.PEX_PYTHON_PATH
+            or interpreter_binary.startswith(ENV.PEX_PYTHON_PATH)
+            or os.path.realpath(interpreter_binary).startswith(
+                tuple(os.path.realpath(p) for p in ENV.PEX_PYTHON_PATH)
+            )
         ):
             interpreter_path = interpreter_binary
     if interpreter_path:
@@ -748,19 +886,25 @@ def venv_dir(
     venv_contents_hash = hashlib.sha1(
         json.dumps(venv_contents, sort_keys=True).encode("utf-8")
     ).hexdigest()
-    venv_path = os.path.join(_expand_pex_root(pex_root), "venvs", pex_hash, venv_contents_hash)
+    pex_root = _expand_pex_root(pex_root) if expand_pex_root else pex_root
+    venv_path = VenvDir.create(pex_hash, venv_contents_hash, pex_root=pex_root)
 
     def warn(message):
         # type: (str) -> None
+
+        if not pex_file:
+            return
+
         from pex.pex_info import PexInfo
 
         pex_warnings.configure_warnings(ENV, PexInfo.from_pex(pex_file))
         pex_warnings.warn(message)
 
     if (
-        ENV.PEX_PYTHON
+        pex_file
+        and ENV.PEX_PYTHON
         and not precise_pex_python
-        and not re.match(r".*[^\d][\d]+\.[\d+]$", ENV.PEX_PYTHON)
+        and not re.match(r".*[^\d][\d]+\.[\d]+$", ENV.PEX_PYTHON)
     ):
         warn(
             dedent(
@@ -782,7 +926,7 @@ def venv_dir(
                 )
             )
         )
-    if not interpreter_path and ENV.PEX_PYTHON_PATH:
+    if pex_file and not interpreter_path and ENV.PEX_PYTHON_PATH:
         warn(
             dedent(
                 """\
@@ -797,7 +941,7 @@ def venv_dir(
                 with PEX_EMIT_WARNINGS=False.
                 """
             ).format(
-                ppp=ENV.PEX_PYTHON_PATH,
+                ppp=os.pathsep.join(ENV.PEX_PYTHON_PATH),
                 pex_file=os.path.normpath(pex_file),
                 venv_path=venv_path,
             )
