@@ -179,8 +179,9 @@ def boot(
             "_PEX_USE_PIP_CONFIG",
             # This is used by Pex's Pip to inject runtime patches dynamically.
             "_PEX_PIP_RUNTIME_PATCHES_PACKAGE",
-            # These are used by Pex's Pip venv to provide foreign platform support and work
-            # around https://github.com/pypa/pip/issues/10050.
+            # These are used by Pex's Pip venv to provide foreign platform support, universal lock
+            # support and work around https://github.com/pypa/pip/issues/10050.
+            "_PEX_INTERPRETER_IMPLEMENTATION",
             "_PEX_PATCHED_MARKERS_FILE",
             "_PEX_PATCHED_TAGS_FILE",
             # These are used by Pex's Pip venv to implement universal locks.
@@ -188,6 +189,8 @@ def boot(
             "_PEX_TARGET_SYSTEMS_FILE",
             # This is used to implement Pex --exclude and --override support.
             "_PEX_DEP_CONFIG_FILE",
+            # This is used to implement --source support.
+            "_PEX_REPOS_CONFIG_FILE",
             # This is used as an experiment knob for atomic_directory locking.
             "_PEX_FILE_LOCK_STYLE",
             # This is used in the scie binding command for ZIPAPP PEXes.
