@@ -1,23 +1,28 @@
-# Copyright 2021 Pants project contributors (see CONTRIBUTORS.md).
+# Copyright 2021 Pex project contributors.
 # Licensed under the Apache License, Version 2.0 (see LICENSE).
+
 import os
 import sys
 from textwrap import dedent
 
-from pex.common import temporary_dir
+import pytest
+
+from pex.common import safe_open, temporary_dir
 from pex.interpreter import PythonInterpreter
+from pex.interpreter_constraints import InterpreterConstraints
 from pex.pex_info import PexInfo
-from pex.testing import (
-    PY27,
-    PY37,
+from pex.pip.version import PipVersion
+from pex.typing import TYPE_CHECKING
+from testing import (
+    PY39,
     PY310,
+    PY311,
     ensure_python_interpreter,
     make_env,
     run_pex_command,
     run_simple_pex,
 )
-from pex.third_party import pkg_resources
-from pex.typing import TYPE_CHECKING
+from testing.pip import skip_if_only_vendored_pip_supported
 
 if TYPE_CHECKING:
     from typing import Any
@@ -32,14 +37,17 @@ def test_interpreter_constraints_to_pex_info_py2():
             [
                 "--disable-cache",
                 "--interpreter-constraint=>=2.7,<3",
-                "--interpreter-constraint=>=3.5",
+                "--interpreter-constraint=>=3.5,<3.12",
                 "-o",
                 pex_out_path,
             ]
         )
         res.assert_success()
         pex_info = PexInfo.from_pex(pex_out_path)
-        assert {">=2.7,<3", ">=3.5"} == set(pex_info.interpreter_constraints)
+        assert (
+            InterpreterConstraints.parse(">=2.7,<3", ">=3.5,<3.12")
+            == pex_info.interpreter_constraints
+        )
 
 
 def test_interpreter_constraints_to_pex_info_py3():
@@ -54,64 +62,84 @@ def test_interpreter_constraints_to_pex_info_py3():
         )
         res.assert_success()
         pex_info = PexInfo.from_pex(pex_out_path)
-        assert [">3"] == pex_info.interpreter_constraints
+        assert InterpreterConstraints.parse(">3") == pex_info.interpreter_constraints
 
 
-def test_interpreter_resolution_with_constraint_option():
-    # type: () -> None
-    with temporary_dir() as output_dir:
-        pex_out_path = os.path.join(output_dir, "pex1.pex")
-        res = run_pex_command(
-            ["--disable-cache", "--interpreter-constraint=>=2.7,<3", "-o", pex_out_path]
-        )
-        res.assert_success()
-        pex_info = PexInfo.from_pex(pex_out_path)
-        assert [">=2.7,<3"] == pex_info.interpreter_constraints
+@pytest.fixture
+def satisfiable_interpreter_constraint():
+    # type: () -> str
+    return "=={major}.{minor}.*".format(major=sys.version_info[0], minor=sys.version_info[1])
 
 
-def test_interpreter_resolution_with_multiple_constraint_options():
-    # type: () -> None
+def test_interpreter_resolution_with_constraint_option(satisfiable_interpreter_constraint):
+    # type: (str) -> None
     with temporary_dir() as output_dir:
         pex_out_path = os.path.join(output_dir, "pex1.pex")
         res = run_pex_command(
             [
                 "--disable-cache",
-                "--interpreter-constraint=>=2.7,<3",
-                # Add a constraint that's impossible to satisfy. Because multiple
-                # constraints OR, the interpeter should still resolve to Python 2.7.
-                "--interpreter-constraint=>=500",
+                "--interpreter-constraint",
+                satisfiable_interpreter_constraint,
                 "-o",
                 pex_out_path,
             ]
         )
         res.assert_success()
         pex_info = PexInfo.from_pex(pex_out_path)
-        assert {">=2.7,<3", ">=500"} == set(pex_info.interpreter_constraints)
+        assert (
+            InterpreterConstraints.parse(satisfiable_interpreter_constraint)
+            == pex_info.interpreter_constraints
+        )
+
+
+def test_interpreter_resolution_with_multiple_constraint_options(
+    satisfiable_interpreter_constraint,
+):
+    # type: (str) -> None
+    with temporary_dir() as output_dir:
+        pex_out_path = os.path.join(output_dir, "pex1.pex")
+        res = run_pex_command(
+            [
+                "--disable-cache",
+                "--interpreter-constraint",
+                satisfiable_interpreter_constraint,
+                # Add a constraint that's impossible to satisfy. Because multiple
+                # constraints OR, the interpreter should still resolve the 1st IC.
+                "--interpreter-constraint",
+                ">=500",
+                "-o",
+                pex_out_path,
+            ]
+        )
+        res.assert_success()
+        pex_info = PexInfo.from_pex(pex_out_path)
+        assert (
+            InterpreterConstraints.parse(satisfiable_interpreter_constraint, ">=500")
+            == pex_info.interpreter_constraints
+        )
 
 
 def test_interpreter_resolution_with_pex_python_path():
     # type: () -> None
+
+    py39 = ensure_python_interpreter(PY39)
+    py310 = ensure_python_interpreter(PY310)
+
     with temporary_dir() as td:
         pexrc_path = os.path.join(td, ".pexrc")
         with open(pexrc_path, "w") as pexrc:
-            # set pex python path
-            pex_python_path = ":".join(
-                [ensure_python_interpreter(PY27), ensure_python_interpreter(PY37)]
-            )
-            pexrc.write("PEX_PYTHON_PATH=%s" % pex_python_path)
+            pexrc.write("PEX_PYTHON_PATH={}".format(os.pathsep.join([py39, py310])))
 
         # constraints to build pex cleanly; PPP + pex_bootstrapper.py
         # will use these constraints to override sys.executable on pex re-exec
-        interpreter_constraint1 = ">3" if sys.version_info[0] == 3 else "<3"
-        interpreter_constraint2 = "<3.8" if sys.version_info[0] == 3 else ">=2.7"
+        interpreter_constraint = "==3.9.*" if sys.version_info[:2] == (3, 10) else "==3.10.*"
 
         pex_out_path = os.path.join(td, "pex.pex")
         res = run_pex_command(
             [
                 "--disable-cache",
-                "--rcfile=%s" % pexrc_path,
-                "--interpreter-constraint=%s,%s"
-                % (interpreter_constraint1, interpreter_constraint2),
+                "--rcfile={}".format(pexrc_path),
+                "--interpreter-constraint={}".format(interpreter_constraint),
                 "-o",
                 pex_out_path,
             ]
@@ -122,37 +150,34 @@ def test_interpreter_resolution_with_pex_python_path():
         stdout, rc = run_simple_pex(pex_out_path, stdin=stdin_payload)
 
         assert rc == 0
-        if sys.version_info[0] == 3:
-            assert str(pex_python_path.split(":")[1]).encode() in stdout
+        if sys.version_info[:2] == (3, 10):
+            assert py39 in stdout.decode("utf-8")
         else:
-            assert str(pex_python_path.split(":")[0]).encode() in stdout
+            assert py310 in stdout.decode("utf-8")
 
 
 def test_interpreter_constraints_honored_without_ppp_or_pp(tmpdir):
     # type: (Any) -> None
     # Create a pex with interpreter constraints, but for not the default interpreter in the path.
 
-    py310_path = ensure_python_interpreter(PY310)
-    py37_path = ensure_python_interpreter(PY37)
+    py311_path = ensure_python_interpreter(PY311)
+    py39_path = ensure_python_interpreter(PY39)
 
     pex_out_path = os.path.join(str(tmpdir), "pex.pex")
     env = make_env(
         PEX_IGNORE_RCFILES="1",
         PATH=os.pathsep.join(
             [
-                os.path.dirname(py37_path),
-                os.path.dirname(py310_path),
+                os.path.dirname(py39_path),
+                os.path.dirname(py311_path),
             ]
         ),
     )
     res = run_pex_command(
-        ["--disable-cache", "--interpreter-constraint===%s" % PY310, "-o", pex_out_path], env=env
+        ["--disable-cache", "--interpreter-constraint===%s" % PY311, "-o", pex_out_path], env=env
     )
     res.assert_success()
 
-    # We want to try to run that pex with no environment variables set
-    stdin_payload = b"import sys; print(sys.executable); sys.exit(0)"
-
     stdout, rc = run_simple_pex(
         pex_out_path,
         args=["-c", "import sys; print('.'.join(map(str, sys.version_info[:2])))"],
@@ -163,14 +188,14 @@ def test_interpreter_constraints_honored_without_ppp_or_pp(tmpdir):
     # If the constraints are honored, it will have run python3.10 and not python3.7
     # Without constraints, we would expect it to use python3.7 as it is the minimum interpreter
     # in the PATH.
-    assert b"3.10\n" == stdout
+    assert b"3.11\n" == stdout
 
 
 def test_interpreter_resolution_pex_python_path_precedence_over_pex_python(tmpdir):
     # type: (Any) -> None
 
     pexrc_path = os.path.join(str(tmpdir), ".pexrc")
-    ppp = ":".join(os.path.dirname(ensure_python_interpreter(py)) for py in (PY27, PY37))
+    ppp = os.pathsep.join(os.path.dirname(ensure_python_interpreter(py)) for py in (PY39, PY310))
     with open(pexrc_path, "w") as pexrc:
         # set both PPP and PP
         pexrc.write(
@@ -179,7 +204,7 @@ def test_interpreter_resolution_pex_python_path_precedence_over_pex_python(tmpdi
                 PEX_PYTHON_PATH={ppp}
                 PEX_PYTHON={pp}
                 """.format(
-                    ppp=ppp, pp=ensure_python_interpreter(PY310)
+                    ppp=ppp, pp=ensure_python_interpreter(PY311)
                 )
             )
         )
@@ -190,7 +215,8 @@ def test_interpreter_resolution_pex_python_path_precedence_over_pex_python(tmpdi
             "--disable-cache",
             "--rcfile",
             pexrc_path,
-            "--interpreter-constraint=>3,<3.8",
+            "--interpreter-constraint",
+            ">=3.9,<3.11",
             "-o",
             pex_out_path,
         ]
@@ -221,7 +247,7 @@ def test_interpreter_resolution_pex_python_path_precedence_over_pex_python(tmpdi
         )
     stdout, rc = run_simple_pex(pex_out_path, print_python_version_command)
     assert rc == 0
-    assert b"3.7\n" == stdout
+    assert b"3.9\n" == stdout
 
 
 def test_plain_pex_exec_no_ppp_no_pp_no_constraints():
@@ -242,25 +268,29 @@ def test_plain_pex_exec_no_ppp_no_pp_no_constraints():
 
 def test_pex_exec_with_pex_python_path_only():
     # type: () -> None
+
+    py39 = ensure_python_interpreter(PY39)
+
     with temporary_dir() as td:
         pexrc_path = os.path.join(td, ".pexrc")
         with open(pexrc_path, "w") as pexrc:
             # set pex python path
-            pex_python_path = ":".join(
-                [ensure_python_interpreter(PY27), ensure_python_interpreter(PY310)]
+            pexrc.write(
+                "PEX_PYTHON_PATH={}".format(
+                    os.pathsep.join([py39, ensure_python_interpreter(PY310)])
+                )
             )
-            pexrc.write("PEX_PYTHON_PATH=%s" % pex_python_path)
 
         pex_out_path = os.path.join(td, "pex.pex")
         res = run_pex_command(["--disable-cache", "--rcfile=%s" % pexrc_path, "-o", pex_out_path])
         res.assert_success()
 
-        # test that pex bootstrapper selects lowest version interpreter
-        # in pex python path (python2.7)
+        # test that pex bootstrapper selects the lowest version interpreter
+        # in pex python path (python3.9)
         stdin_payload = b"import sys; print(sys.executable); sys.exit(0)"
         stdout, rc = run_simple_pex(pex_out_path, stdin=stdin_payload)
         assert rc == 0
-        assert str(pex_python_path.split(":")[0]).encode() in stdout
+        assert py39 in stdout.decode("utf-8")
 
 
 def test_pex_exec_with_pex_python_path_and_pex_python_but_no_constraints(tmpdir):
@@ -274,7 +304,9 @@ def test_pex_exec_with_pex_python_path_and_pex_python_but_no_constraints(tmpdir)
                 PEX_PYTHON_PATH={}
                 PEX_PYTHON=python
                 """.format(
-                    ":".join(os.path.dirname(ensure_python_interpreter(py)) for py in (PY310, PY27))
+                    os.pathsep.join(
+                        os.path.dirname(ensure_python_interpreter(py)) for py in (PY310, PY39)
+                    )
                 )
             )
         )
@@ -283,34 +315,34 @@ def test_pex_exec_with_pex_python_path_and_pex_python_but_no_constraints(tmpdir)
     res = run_pex_command(["--disable-cache", "--rcfile", pexrc_path, "-o", pex_out_path])
     res.assert_success()
 
-    # test that pex bootstrapper selects lowest version interpreter
-    # in pex python path (python2.7)
+    # test that pex bootstrapper selects the lowest version interpreter
+    # in pex python path (python3.9)
     stdout, rc = run_simple_pex(
         pex_out_path, args=["-c", "import sys; print('.'.join(map(str, sys.version_info[:2])))"]
     )
     assert rc == 0
-    assert b"2.7\n" == stdout
+    assert b"3.9\n" == stdout
 
 
 def test_pex_python():
     # type: () -> None
-    py2_path_interpreter = ensure_python_interpreter(PY27)
-    py3_path_interpreter = ensure_python_interpreter(PY37)
-    path = ":".join([os.path.dirname(py2_path_interpreter), os.path.dirname(py3_path_interpreter)])
-    env = make_env(PATH=path)
+    py39 = ensure_python_interpreter(PY39)
+    py310 = ensure_python_interpreter(PY310)
+    env = make_env(PATH=os.pathsep.join([os.path.dirname(py39), os.path.dirname(py310)]))
     with temporary_dir() as td:
         pexrc_path = os.path.join(td, ".pexrc")
         with open(pexrc_path, "w") as pexrc:
-            pex_python = ensure_python_interpreter(PY37)
-            pexrc.write("PEX_PYTHON=%s" % pex_python)
+            pexrc.write("PEX_PYTHON={}".format(py39))
 
         # test PEX_PYTHON with valid constraints
         pex_out_path = os.path.join(td, "pex.pex")
         res = run_pex_command(
             [
                 "--disable-cache",
-                "--rcfile=%s" % pexrc_path,
-                "--interpreter-constraint=>3,<3.8",
+                "--rcfile",
+                pexrc_path,
+                "--interpreter-constraint",
+                ">=3.9,<3.11",
                 "-o",
                 pex_out_path,
             ],
@@ -321,21 +353,22 @@ def test_pex_python():
         stdin_payload = b"import sys; print(sys.executable); sys.exit(0)"
         stdout, rc = run_simple_pex(pex_out_path, stdin=stdin_payload, env=env)
         assert rc == 0
-        correct_interpreter_path = pex_python.encode()
-        assert correct_interpreter_path in stdout
+        assert py39 in stdout.decode("utf-8")
 
         # test PEX_PYTHON with incompatible constraints
+        py310 = ensure_python_interpreter(PY310)
         pexrc_path = os.path.join(td, ".pexrc")
         with open(pexrc_path, "w") as pexrc:
-            pex_python = ensure_python_interpreter(PY27)
-            pexrc.write("PEX_PYTHON=%s" % pex_python)
+            pexrc.write("PEX_PYTHON={}".format(py310))
 
         pex_out_path = os.path.join(td, "pex2.pex")
         res = run_pex_command(
             [
                 "--disable-cache",
-                "--rcfile=%s" % pexrc_path,
-                "--interpreter-constraint=>3,<3.8",
+                "--rcfile",
+                pexrc_path,
+                "--interpreter-constraint",
+                ">=3.8,<3.10",
                 "-o",
                 pex_out_path,
             ],
@@ -346,117 +379,126 @@ def test_pex_python():
         stdin_payload = b"import sys; print(sys.executable); sys.exit(0)"
         stdout, rc = run_simple_pex(pex_out_path, stdin=stdin_payload, env=env)
         assert rc == 1
-        fail_str = ("Failed to find a compatible PEX_PYTHON={}.".format(pex_python)).encode()
-        assert fail_str in stdout
+        assert "Failed to find a compatible PEX_PYTHON={}.".format(py310) in stdout.decode("utf-8")
 
         # test PEX_PYTHON with no constraints
         pex_out_path = os.path.join(td, "pex3.pex")
         res = run_pex_command(
-            ["--disable-cache", "--rcfile=%s" % pexrc_path, "-o", pex_out_path], env=env
+            ["--disable-cache", "--rcfile", pexrc_path, "-o", pex_out_path], env=env
         )
         res.assert_success()
 
         stdin_payload = b"import sys; print(sys.executable); sys.exit(0)"
         stdout, rc = run_simple_pex(pex_out_path, stdin=stdin_payload, env=env)
         assert rc == 0
-        correct_interpreter_path = pex_python.encode()
-        assert correct_interpreter_path in stdout
+        assert py310 in stdout.decode("utf-8")
 
 
-def test_interpreter_selection_using_os_environ_for_bootstrap_reexec():
-    # type: () -> None
+@skip_if_only_vendored_pip_supported
+def test_interpreter_selection_using_os_environ_for_bootstrap_reexec(
+    tmpdir,  # type: Any
+    pex_project_dir,  # type: str
+):
+    # type: (...) -> None
     """This is a test for verifying the proper function of the pex bootstrapper's interpreter
     selection logic and validate a corresponding bugfix.
 
     More details on the nature of the bug can be found at:
-    https://github.com/pantsbuild/pex/pull/441
+    https://github.com/pex-tool/pex/pull/441
     """
-    with temporary_dir() as td:
-        pexrc_path = os.path.join(td, ".pexrc")
+    td = os.path.join(str(tmpdir), "tester_project")
+    pexrc_path = os.path.join(td, ".pexrc")
 
-        # Select pexrc interpreter versions based on test environment.
-        # The parent interpreter is the interpreter we expect the parent pex to
-        # execute with. The child interpreter is the interpreter we expect the
-        # child pex to execute with.
-        if sys.version_info[:2] == (3, 8):
-            child_pex_interpreter_version = PY310
-        else:
-            child_pex_interpreter_version = PY27
+    # Select pexrc interpreter versions based on test environment.
+    # The parent interpreter is the interpreter we expect the parent pex to
+    # execute with. The child interpreter is the interpreter we expect the
+    # child pex to execute with.
+    if sys.version_info[:2] == (3, 10):
+        child_pex_interpreter_version = PY39
+    else:
+        child_pex_interpreter_version = PY310
 
-        # Write parent pex's pexrc.
-        with open(pexrc_path, "w") as pexrc:
-            pexrc.write("PEX_PYTHON=%s" % sys.executable)
+    # Write parent pex's pexrc.
+    with safe_open(pexrc_path, "w") as pexrc:
+        pexrc.write("PEX_PYTHON={}".format(sys.executable))
 
-        # The code below depends on pex.testing which depends on pytest - make sure the built pex gets
-        # this dep.
-        pytest_dist = pkg_resources.WorkingSet().find(pkg_resources.Requirement.parse("pytest"))
+    test_setup_path = os.path.join(td, "setup.py")
+    with safe_open(test_setup_path, "w") as fh:
+        fh.write(
+            dedent(
+                """
+                from setuptools import setup
 
-        test_setup_path = os.path.join(td, "setup.py")
-        with open(test_setup_path, "w") as fh:
-            fh.write(
-                dedent(
-                    """
-                    from setuptools import setup
 
-                    setup(
-                        name='tester',
-                        version='1.0',
-                        description='tests',
-                        author='tester',
-                        author_email='test@test.com',
-                        packages=['testing'],
-                        install_requires={install_requires!r}
-                    )
-                    """.format(
-                        install_requires=[str(pytest_dist.as_requirement())]
-                    )
+                setup(
+                    name="tester",
+                    version="1.0",
+                    description="tests",
+                    author="tester",
+                    author_email="tester@test.com",
+                    packages=["tester"],
                 )
+                """
             )
-
-        os.mkdir(os.path.join(td, "testing"))
-        test_init_path = os.path.join(td, "testing/__init__.py")
-        with open(test_init_path, "w") as fh:
-            fh.write(
-                dedent(
-                    '''
-                    def tester():
-                        from pex.testing import (
-                            run_pex_command,
-                            run_simple_pex,
-                            temporary_dir
-                        )
-                        import os
-                        from textwrap import dedent
-                        with temporary_dir() as td:
-                            pexrc_path = os.path.join(td, '.pexrc')
-                            with open(pexrc_path, 'w') as pexrc:
-                                pexrc.write("PEX_PYTHON={}")
-                            test_file_path = os.path.join(td, 'build_and_run_child_pex.py')
-                            with open(test_file_path, 'w') as fh:
-                                fh.write(dedent("""
-                                    import sys
-                                    print(sys.executable)
-                                """))
-                            pex_out_path = os.path.join(td, 'child.pex')
-                            res = run_pex_command(['--disable-cache',
-                                '-o', pex_out_path])
-                            stdin_payload = b'import sys; print(sys.executable); sys.exit(0)'
-                            stdout, rc = run_simple_pex(pex_out_path, stdin=stdin_payload)
-                            print(stdout)
-                    '''.format(
-                        ensure_python_interpreter(child_pex_interpreter_version)
-                    )
-                )
-            )
-
-        pex_out_path = os.path.join(td, "parent.pex")
-        res = run_pex_command(
-            ["--disable-cache", "pex", "{}".format(td), "-e", "testing:tester", "-o", pex_out_path]
         )
-        res.assert_success()
 
-        stdout, rc = run_simple_pex(pex_out_path)
-        assert rc == 0
-        # Ensure that child pex used the proper interpreter as specified by its pexrc.
-        correct_interpreter_path = ensure_python_interpreter(child_pex_interpreter_version)
-        assert correct_interpreter_path in stdout.decode("utf-8")
+    test_init_path = os.path.join(td, "tester/__init__.py")
+    with safe_open(test_init_path, "w") as fh:
+        fh.write(
+            dedent(
+                """\
+                def test_it():
+                    import atexit
+                    import os
+                    import shutil
+                    import subprocess
+                    import sys
+                    import tempfile
+
+
+                    td = tempfile.mkdtemp()
+                    atexit.register(shutil.rmtree, td)
+
+                    pexrc_path = os.path.join(td, ".pexrc")
+                    with open(pexrc_path, "w") as pexrc:
+                        pexrc.write("PEX_PYTHON={}")
+
+                    pex_out_path = os.path.join(td, "child.pex")
+                    subprocess.check_call(
+                        [sys.executable, "-mpex", "--disable-cache", "-o", pex_out_path]
+                    )
+                    process = subprocess.Popen(
+                        [sys.executable, pex_out_path, "-c", "import sys; print(sys.executable)"],
+                        stdout=subprocess.PIPE,
+                        stderr=subprocess.STDOUT,
+                    )
+                    stdout, _ = process.communicate()
+                    print(stdout)
+                    sys.exit(process.returncode)
+                """.format(
+                    ensure_python_interpreter(child_pex_interpreter_version)
+                )
+            )
+        )
+
+    pex_out_path = os.path.join(td, "parent.pex")
+    res = run_pex_command(
+        [
+            "--disable-cache",
+            "--pip-version",
+            PipVersion.LATEST_COMPATIBLE.value,
+            pex_project_dir,
+            td,
+            "-e",
+            "tester:test_it",
+            "-o",
+            pex_out_path,
+        ]
+    )
+    res.assert_success()
+
+    stdout, rc = run_simple_pex(pex_out_path)
+    assert rc == 0, stdout
+    # Ensure that child pex used the proper interpreter as specified by its pexrc.
+    correct_interpreter_path = ensure_python_interpreter(child_pex_interpreter_version)
+    assert correct_interpreter_path in stdout.decode("utf-8")
