@@ -1,15 +1,14 @@
 # coding=utf-8
-# Copyright 2021 Pants project contributors (see CONTRIBUTORS.md).
+# Copyright 2021 Pex project contributors.
 # Licensed under the Apache License, Version 2.0 (see LICENSE).
 
 import os.path
 import platform
-import subprocess
 import sys
 
 from pex.common import safe_rmtree
-from pex.testing import make_env, run_pex_command
 from pex.typing import TYPE_CHECKING
+from testing import make_env, run_pex_command, subprocess
 
 if TYPE_CHECKING:
     from typing import Any
