@@ -10,8 +10,9 @@ from pex import dist_metadata, resolver, targets
 from pex.fs import safe_symlink
 from pex.pip.tool import PackageIndexConfiguration
 from pex.resolve.configured_resolver import ConfiguredResolver
-from pex.resolve.locked_resolve import LockConfiguration, LockedResolve, LockStyle
+from pex.resolve.locked_resolve import LockedResolve, LockStyle
 from pex.resolve.lockfile.create import LockObserver
+from pex.resolve.package_repository import Repo, ReposConfiguration
 from pex.resolve.resolved_requirement import Pin
 from pex.resolve.resolver_configuration import PipConfiguration
 from pex.resolver import Downloaded, LocalDistribution, WheelBuilder
@@ -41,18 +42,17 @@ def normalize(
     )
 
 
-def create_lock_observer(lock_configuration):
-    # type: (LockConfiguration) -> LockObserver
+def create_lock_observer(lock_style):
+    # type: (LockStyle.Value) -> LockObserver
     pip_configuration = PipConfiguration()
     package_index_configuration = PackageIndexConfiguration.create(
         resolver_version=pip_configuration.resolver_version,
-        indexes=pip_configuration.repos_configuration.indexes,
-        find_links=pip_configuration.repos_configuration.find_links,
+        repos_configuration=pip_configuration.repos_configuration,
         network_configuration=pip_configuration.network_configuration,
     )
     return LockObserver(
         root_requirements=(),
-        lock_configuration=lock_configuration,
+        lock_style=lock_style,
         resolver=ConfiguredResolver(pip_configuration=pip_configuration),
         wheel_builder=WheelBuilder(
             package_index_configuration,
@@ -63,11 +63,11 @@ def create_lock_observer(lock_configuration):
 
 
 def create_lock(
-    lock_configuration,  # type: LockConfiguration
+    lock_style,  # type: LockStyle.Value
     **kwargs  # type: Any
 ):
     # type: (...) -> Tuple[Downloaded, Tuple[LockedResolve, ...]]
-    lock_observer = create_lock_observer(lock_configuration)
+    lock_observer = create_lock_observer(lock_style)
     downloaded = resolver.download(
         observer=lock_observer,
         resolver=ConfiguredResolver(pip_configuration=PipConfiguration()),
@@ -86,20 +86,17 @@ def create_lock(
     ),
 )
 @pytest.mark.parametrize(
-    "lock_configuration",
-    (
-        pytest.param(LockConfiguration(style=LockStyle.STRICT), id="strict"),
-        pytest.param(LockConfiguration(style=LockStyle.SOURCES), id="sources"),
-    ),
+    "lock_style",
+    [pytest.param(style, id=str(style)) for style in (LockStyle.STRICT, LockStyle.SOURCES)],
 )
 def test_lock_single_target(
     tmpdir,  # type: Any
     requirements,  # type: Iterable[str]
-    lock_configuration,  # type: LockConfiguration
+    lock_style,  # type: LockStyle.Value
 ):
     # type: (...) -> None
 
-    downloaded, locked_resolves = create_lock(lock_configuration, requirements=requirements)
+    downloaded, locked_resolves = create_lock(lock_style, requirements=requirements)
     assert 1 == len(locked_resolves)
     lock = locked_resolves[0]
 
@@ -139,10 +136,12 @@ def test_lock_single_target(
             local_dist.path, os.path.join(find_links_repo, os.path.basename(local_dist.path))
         )
     _, find_links_locked_resolves = create_lock(
-        lock_configuration,
+        lock_style,
         requirements=requirements,
-        indexes=[],
-        find_links=[find_links_repo],
+        repos_configuration=ReposConfiguration.create(
+            indexes=[],
+            find_links=[Repo(find_links_repo)],
+        ),
     )
     assert normalize(
         locked_resolves, skip_additional_artifacts=True, skip_urls=True, skip_verified=True
