@@ -1,11 +1,12 @@
-# Copyright 2014 Pants project contributors (see CONTRIBUTORS.md).
+# Copyright 2014 Pex project contributors.
 # Licensed under the Apache License, Version 2.0 (see LICENSE).
 
+from __future__ import absolute_import
+
 import glob
 import json
 import os
 import re
-import subprocess
 import sys
 from collections import defaultdict
 from contextlib import contextmanager
@@ -13,27 +14,30 @@ from textwrap import dedent
 
 import pytest
 
-from pex import interpreter
-from pex.common import chmod_plus_x, safe_mkdir, safe_mkdtemp, temporary_dir, touch
-from pex.compatibility import PY3
+from pex.cache.dirs import InterpreterDir
+from pex.common import environment_as, safe_mkdir, safe_mkdtemp, temporary_dir, touch
+from pex.executables import chmod_plus_x
 from pex.executor import Executor
-from pex.interpreter import PythonInterpreter
+from pex.fs import safe_rename, safe_symlink
+from pex.interpreter import PythonInterpreter, create_shebang
 from pex.jobs import Job
 from pex.pyenv import Pyenv
-from pex.testing import (
-    ALL_PY_VERSIONS,
+from pex.typing import TYPE_CHECKING, cast
+from pex.variables import ENV
+from testing import (
     PY27,
-    PY37,
+    PY39,
     PY310,
+    PY311,
     PY_VER,
+    all_pythons,
     ensure_python_distribution,
     ensure_python_interpreter,
     ensure_python_venv,
-    environment_as,
     pushd,
+    subprocess,
 )
-from pex.typing import TYPE_CHECKING
-from pex.variables import ENV
+from testing.pytest_utils.tmp import TempdirFactory
 
 try:
     from unittest.mock import Mock, patch  # type: ignore[import]
@@ -55,19 +59,13 @@ class TestPythonInterpreter(object):
     def test_all_does_not_raise_with_empty_path_envvar(self):
         # type: () -> None
         """additionally, tests that the module does not raise at import."""
-        with patch.dict(os.environ, clear=True):
-            if PY3:
-                import importlib
-
-                importlib.reload(interpreter)
-            else:
-                reload(interpreter)
+        with patch.dict(os.environ, clear=True), PythonInterpreter._cleared_memory_cache():
             PythonInterpreter.all()
 
-    TEST_INTERPRETER1_VERSION = PY27
+    TEST_INTERPRETER1_VERSION = PY310
     TEST_INTERPRETER1_VERSION_TUPLE = tuple_from_version(TEST_INTERPRETER1_VERSION)
 
-    TEST_INTERPRETER2_VERSION = PY37
+    TEST_INTERPRETER2_VERSION = PY39
     TEST_INTERPRETER2_VERSION_TUPLE = tuple_from_version(TEST_INTERPRETER2_VERSION)
 
     @pytest.fixture
@@ -124,7 +122,7 @@ class TestPythonInterpreter(object):
             "python3.123m",
         )
 
-        matches = PythonInterpreter._matches_binary_name
+        matches = PythonInterpreter.matches_binary_name
         for name in valid_binary_names:
             assert matches(name), "Expected {} to be valid binary name".format(name)
 
@@ -200,8 +198,13 @@ class TestPythonInterpreter(object):
 
     def test_iter_interpreter_path_filter_symlink(self, test_interpreter1, test_interpreter2):
         # type: (str, str) -> None
-        with temporary_dir() as bin_dir:
-            os.symlink(test_interpreter2, os.path.join(bin_dir, "jake"))
+        with temporary_dir() as tmpdir:
+            # PythonInterpreter._find_pyvenv_cfg looks in the binary's parent dir for a
+            # pyvenv.cfg file. We use a subdir to ensure that a random such file in /tmp
+            # doesn't impact this test.
+            bin_dir = os.path.join(tmpdir, "bin_dir")
+            os.mkdir(bin_dir)
+            safe_symlink(test_interpreter2, os.path.join(bin_dir, "jake"))
 
             # Verify path filtering happens before interpreter resolution, which os.path.realpaths
             # the interpreter binary. This supports specifying a path filter like
@@ -218,8 +221,11 @@ class TestPythonInterpreter(object):
 
     def test_pyenv_shims(self, tmpdir):
         # type: (Any) -> None
-        py37, _, run_pyenv = ensure_python_distribution(PY37)
-        py310 = ensure_python_interpreter(PY310)
+        py39_dist = ensure_python_distribution(PY39)
+        py39 = py39_dist.binary
+        run_pyenv = py39_dist.run_pyenv
+
+        py311 = ensure_python_interpreter(PY311)
 
         pyenv_root = str(run_pyenv(["root"]).strip())
         pyenv_shims = os.path.join(pyenv_root, "shims")
@@ -241,7 +247,7 @@ class TestPythonInterpreter(object):
         pex_root = os.path.join(str(tmpdir), "pex_root")
         cwd = safe_mkdir(os.path.join(str(tmpdir), "home", "jake", "project"))
         with ENV.patch(PEX_ROOT=pex_root) as pex_env, environment_as(
-            PYENV_ROOT=pyenv_root, PEX_PYTHON_PATH=pyenv_shims, **pex_env
+            **dict(pex_env, PYENV_ROOT=pyenv_root, PEX_PYTHON_PATH=pyenv_shims)
         ), pyenv_shell(), pushd(cwd):
             pyenv = Pyenv.find()
             assert pyenv is not None
@@ -265,49 +271,49 @@ class TestPythonInterpreter(object):
                 with pytest.raises(PythonInterpreter.IdentificationError):
                     interpreter_for_shim(shim_name)
 
-            pyenv_global(PY37, PY310)
-            assert_shim("python", py37)
-            assert_shim("python3", py37)
-            assert_shim("python3.7", py37)
-            assert_shim("python3.10", py310)
+            pyenv_global(PY39, PY311)
+            assert_shim("python", py39)
+            assert_shim("python3", py39)
+            assert_shim("python3.9", py39)
+            assert_shim("python3.11", py311)
 
-            pyenv_global(PY310, PY37)
-            assert_shim("python", py310)
-            assert_shim("python3", py310)
-            assert_shim("python3.10", py310)
-            assert_shim("python3.7", py37)
+            pyenv_global(PY311, PY39)
+            assert_shim("python", py311)
+            assert_shim("python3", py311)
+            assert_shim("python3.11", py311)
+            assert_shim("python3.9", py39)
 
-            pyenv_local(PY37)
-            assert_shim("python", py37)
-            assert_shim("python3", py37)
-            assert_shim("python3.7", py37)
-            assert_shim_inactive("python3.8")
+            pyenv_local(PY39)
+            assert_shim("python", py39)
+            assert_shim("python3", py39)
+            assert_shim("python3.9", py39)
+            assert_shim_inactive("python3.7")
 
-            with pyenv_shell(PY310):
-                assert_shim("python", py310)
-                assert_shim("python3", py310)
-                assert_shim("python3.10", py310)
-                assert_shim_inactive("python3.7")
+            with pyenv_shell(PY311):
+                assert_shim("python", py311)
+                assert_shim("python3", py311)
+                assert_shim("python3.11", py311)
+                assert_shim_inactive("python3.9")
 
-            with pyenv_shell(PY37, PY310):
-                assert_shim("python", py37)
-                assert_shim("python3", py37)
-                assert_shim("python3.7", py37)
-                assert_shim("python3.10", py310)
+            with pyenv_shell(PY39, PY311):
+                assert_shim("python", py39)
+                assert_shim("python3", py39)
+                assert_shim("python3.9", py39)
+                assert_shim("python3.11", py311)
 
-            # The shim pointer is now invalid since python3.7 was uninstalled and so
+            # The shim pointer is now invalid since python3.9 was uninstalled and so
             # should be re-read and found invalid.
-            py37_version_dir = os.path.dirname(os.path.dirname(py37))
-            py37_deleted = "{}.uninstalled".format(py37_version_dir)
-            os.rename(py37_version_dir, py37_deleted)
+            py39_version_dir = os.path.dirname(os.path.dirname(py39))
+            py39_deleted = "{}.uninstalled".format(py39_version_dir)
+            safe_rename(py39_version_dir, py39_deleted)
             try:
                 assert_shim_inactive("python")
                 assert_shim_inactive("python3")
-                assert_shim_inactive("python3.7")
+                assert_shim_inactive("python3.9")
             finally:
-                os.rename(py37_deleted, py37_version_dir)
+                safe_rename(py39_deleted, py39_version_dir)
 
-            assert_shim("python", py37)
+            assert_shim("python", py39)
 
 
 def test_latest_release_of_min_compatible_version():
@@ -334,8 +340,8 @@ def test_latest_release_of_min_compatible_version():
 def test_detect_pyvenv(tmpdir):
     # type: (Any) -> None
     venv = str(tmpdir)
-    py37 = ensure_python_interpreter(PY37)
-    real_interpreter = PythonInterpreter.from_binary(py37)
+    py39 = ensure_python_interpreter(PY39)
+    real_interpreter = PythonInterpreter.from_binary(py39)
     real_interpreter.execute(["-m", "venv", venv])
     with pytest.raises(Executor.NonZeroExit):
         real_interpreter.execute(["-c", "import colors"])
@@ -354,7 +360,7 @@ def test_detect_pyvenv(tmpdir):
     ), "Expected exactly one canonical venv python, found: {}".format(canonical_to_python)
     canonical, pythons = canonical_to_python.popitem()
 
-    real_python = os.path.realpath(py37)
+    real_python = os.path.realpath(py39)
     assert canonical != real_python
     assert os.path.dirname(canonical) == venv_bin_dir
     assert os.path.realpath(canonical) == real_python
@@ -373,7 +379,11 @@ def check_resolve_venv(real_interpreter):
     ):
         # type: (...) -> List[str]
         venv_dir = os.path.join(tmpdir, rel_path)
-        interpreter.execute(["-m", "venv", venv_dir])
+
+        # N.B.: We don't need pip in the venv for this test and sometimes system interpreters
+        # don't have ensurepip support.
+        interpreter.execute(["-m", "venv", "--without-pip", venv_dir])
+
         return glob.glob(os.path.join(venv_dir, "bin", "python*"))
 
     assert not real_interpreter.is_venv
@@ -395,7 +405,7 @@ def check_resolve_venv(real_interpreter):
 
 def test_resolve_venv():
     # type: () -> None
-    real_interpreter = PythonInterpreter.from_binary(ensure_python_interpreter(PY37))
+    real_interpreter = PythonInterpreter.from_binary(ensure_python_interpreter(PY39))
     check_resolve_venv(real_interpreter)
 
 
@@ -411,29 +421,32 @@ def test_resolve_venv_ambient():
 def test_identify_cwd_isolation_issues_1231(tmpdir):
     # type: (Any) -> None
 
-    python37, pip = ensure_python_venv(PY37)
+    venv = ensure_python_venv(PY27)
+    python27 = venv.interpreter.binary
+    pip = venv.bin_path("pip")
+
     polluted_cwd = os.path.join(str(tmpdir), "dir")
     subprocess.check_call(args=[pip, "install", "--target", polluted_cwd, "pex==2.1.16"])
 
     pex_root = os.path.join(str(tmpdir), "pex_root")
-    with pushd(polluted_cwd), ENV.patch(PEX_ROOT=pex_root):
-        interp = PythonInterpreter.from_binary(python37)
+    with pushd(polluted_cwd), ENV.patch(
+        PEX_ROOT=pex_root
+    ), PythonInterpreter._cleared_memory_cache():
+        interp = PythonInterpreter.from_binary(python27)
 
-    interp_info_files = {
-        os.path.join(root, f)
-        for root, _, files in os.walk(pex_root)
-        for f in files
-        if f == PythonInterpreter.INTERP_INFO_FILE
-    }
-    assert 1 == len(interp_info_files)
-    with open(interp_info_files.pop()) as fp:
+    interpreter_dirs = list(InterpreterDir.iter_all(pex_root=pex_root))
+    assert 1 == len(interpreter_dirs)
+    with open(interpreter_dirs[0].interp_info_file) as fp:
         assert interp.binary == json.load(fp)["binary"]
 
 
 @pytest.fixture(scope="module")
-def macos_monterey_interpeter(tmpdir_factory):
-    # type: (Any) -> str
-    pythonwrapper = os.path.join(str(tmpdir_factory.mktemp("bin")), "pythonwrapper")
+def macos_monterey_interpeter(
+    tmpdir_factory,  # type: TempdirFactory
+    request,  # type: Any
+):
+    # type: (...) -> str
+    pythonwrapper = tmpdir_factory.mktemp("bin", request=request).join("pythonwrapper")
     with open(pythonwrapper, "w") as fp:
         fp.write(
             dedent(
@@ -447,8 +460,8 @@ def macos_monterey_interpeter(tmpdir_factory):
         )
     chmod_plus_x(pythonwrapper)
 
-    python = os.path.join(str(tmpdir_factory.mktemp("bin")), "python")
-    os.symlink(pythonwrapper, python)
+    python = tmpdir_factory.mktemp("bin", request=request).join("python")
+    safe_symlink(pythonwrapper, python)
     return python
 
 
@@ -493,13 +506,94 @@ def test_issue_1494_iter_candidates(macos_monterey_interpeter):
     ] == list(PythonInterpreter.iter_candidates(paths=[sys.executable, macos_monterey_interpeter]))
 
 
-@pytest.mark.parametrize("py_version", ALL_PY_VERSIONS)
-def test_sys_path(py_version):
+@pytest.mark.parametrize(
+    "python",
+    [
+        pytest.param(python, id=str(PythonInterpreter.from_binary(python).identity))
+        for python in all_pythons()
+    ],
+)
+def test_sys_path(python):
     # type: (str) -> None
 
-    interp = PythonInterpreter.from_binary(ensure_python_interpreter(py_version))
-    _, stdout, _ = interp.execute(args=["-c", "import os, sys; print(os.linesep.join(sys.path))"])
+    interp = PythonInterpreter.from_binary(python)
+    _, stdout, _ = interp.execute(args=["-c", r"import sys; print('\n'.join(sys.path))"])
     assert tuple(entry for entry in stdout.splitlines() if entry) == interp.sys_path, (
         'Its expected the sys_path matches the runtime sys.path with the exception of the PWD ("") '
         "head entry."
     )
+
+
+def test_sys_path_leak_for_current(tmpdir):
+    # type: (Any) -> None
+
+    expected_sys_path = [
+        entry
+        for entry in json.loads(
+            subprocess.check_output(
+                args=[
+                    sys.executable,
+                    "-sE",
+                    "-c",
+                    "import json, sys; json.dump(sys.path, sys.stdout)",
+                ]
+            ).decode("utf-8")
+        )
+        # N.B.: We expect the PythonInterpreter infrastructure to elide the implicit "" / CWD
+        # sys.path entry.
+        if entry
+    ]
+    pex_root = os.path.join(str(tmpdir), "pex_root")
+
+    new_sys_path_entry = os.path.join(str(tmpdir), "not-interpreter-sys-path")
+    with ENV.patch(PEX_ROOT=pex_root), PythonInterpreter._cleared_memory_cache():
+        sys.path.append(new_sys_path_entry)
+        try:
+            assert tuple(expected_sys_path) == PythonInterpreter.get().sys_path
+        finally:
+            sys.path.pop()
+
+
+def test_create_shebang(tmpdir):
+    # type: (Any) -> None
+
+    python_exe = sys.executable
+
+    def execute_script(
+        max_shebang_length,  # type: int
+        *extra_args  # type: str
+    ):
+        # type: (...) -> str
+        with open(os.path.join(str(tmpdir), "script.py"), "w") as fp:
+            fp.write(
+                dedent(
+                    """\
+                    {shebang}
+
+                    import json
+                    import sys
+
+
+                    with open(sys.argv[0]) as fp:
+                        json.dump(dict(shebang=fp.readline(), argv=sys.argv), sys.stdout)
+                    """
+                ).format(
+                    shebang=create_shebang(
+                        python_exe=python_exe, max_shebang_length=max_shebang_length
+                    )
+                )
+            )
+        chmod_plus_x(fp.name)
+        result = json.loads(
+            subprocess.check_output(args=[fp.name] + list(extra_args)).decode("utf-8")
+        )
+        assert [fp.name] + list(extra_args) == result["argv"]
+        return cast(str, result["shebang"])
+
+    assert "#!{python_exe}\n".format(python_exe=python_exe) == execute_script(len(python_exe) * 2)
+    assert "#!{python_exe}\n".format(python_exe=python_exe) == execute_script(
+        len(python_exe) * 2, "foo"
+    )
+
+    assert "#!/bin/sh\n" == execute_script(len(python_exe) // 2)
+    assert "#!/bin/sh\n" == execute_script(len(python_exe) // 2, "bar", "baz")
